// mmc.c : MultiMediaCard functions for SPI Mode.
//
//      nobodyo<nospam>@</nospam>web.de
//
// V 0.01       7. April 2003

// MMC Lib 
#ifndef _MMCLIB_C
#define _MMCLIB_C

//---------------------------------------------------------------------

#include "mmc.h"
#include  "MSP430x14x.H"
#include  "math.h"
#include  "string.h"



//---------------------------------------------------------------------




// setup usart1 in spi mode
void
uart1_spi_mode_init (void)
{
  ME2 |= USPIE1;		// Enable USART1 SPI mode
  UTCTL1 = CKPH | SSEL1 | SSEL0 | STC;	// SMCLK, 3-pin mode ( CKPL is CKPH for IAR Compiler)
  UCTL1 = CHAR | SYNC | MM;	// 8-bit SPI Master **SWRST**
  UBR01 = 0x02;			// UCLK/2 (2,4 MHz)
  UBR11 = 0x00;			// 0
  UMCTL1 = 0x00;		// no modulation
  P5SEL |= 0x0E;		// P5.1-3 SPI option select
  P5DIR |= 0x01;		// P5.0 output direction
  P5OUT = 0x00;
  return;
}


// Chip Select of the MMC (P5.4 = 1) means deactivation of the card
void
mmc_cs (void)
{
  P5OUT |= 0x10;
  return;
}

// Chip Deselect of the MMC (P5.4 = 0) means activation of the card
void
mmc_cds (void)
{
  while (!(U1TCTL & TXEPT));	// wait for TX empty
  P5OUT &= ~0x10;
  return;
}


// Initialisieren
char
mmc_init (void)
{
  char rValue = MMC_INIT_ERROR;
  unsigned int i = 0;

  // setup the SPI:
  // clock idle LOW 
  // data valid on rising edge 
  // setup the usart
  uart1_spi_mode_init ();

  // begin MMC initialization sequence 
  // card must be clock at least 74 times (this block sends 80 clocks) 
  // chip select mmc
  mmc_cs ();
  // send DUMMY (0xFF) 10 times 
  for (i = 0; i < 10; ++i)
    {
      spi_write (DUMMY);
      while ((IFG2 & UTXIFG1) == 0);
    }

  mmc_cds ();

  // command to put MMC into SPI mode (reset command, CMD0) 
  // command = 0x40 
  // all args = 0x00 
  // crc = 0x95 (pre-calculated) 
  mmc_send_command (0x40, 0x00, 0x00, 0x00, 0x00, 0x95);

  // send commands to bring card out of idle state 
  if (mmc_wait_for_response (0x01) == MMC_SUCCESS)
    {
      i = 0xFF;
      do
	{
	  // MMC CMD1
	  mmc_send_command (0x41, 0x00, 0x00, 0x00, 0x00, 0xFF);
	  --i;			// decrement counter
	}
      while (mmc_wait_for_response (MMC_R1_RESPONSE) != MMC_SUCCESS && i > 0);

      // initialization fails if the card didn't come out of idle state 
      if (i != 0)
	{
	  rValue = MMC_SUCCESS;	// card came out of idle state  w / out timing out
	}
    }

  // SS = HIGH (off) 
  mmc_cs ();

  // give the MMC the required clocks to finish up what ever it needs to do 
  spi_write (0xFF);

  return rValue;		// 0 if ok, 4 if init error
}




//---------------------------------------------------------------------
// Wait for expected value (0x00, 0x01, 0xFE) or timeout after 255 reads
char
mmc_wait_for_response (char expected_c)
{
  char rValue = MMC_SUCCESS;
  unsigned char val;
  unsigned char count = 0xFF;

  do
    {
      val = spi_read ();
    }
  while (val != expected_c && --count > 0);

  if (count == 0)		// timed-out while waiting for a valid response 
    {
      rValue = MMC_TIMEOUT_ERROR;	// 0xFF 
    }

  return rValue;
}



//---------------------------------------------------------------------
void
mmc_read_block (unsigned long int address, unsigned char count)
{
  unsigned long i = 0;
  char rvalue = MMC_SUCCESS;	// 0

  // Set the block length to read 
  if (mmc_set_block_length (count))
    {
      // SS = LOW (on) 
      mmc_cds ();

      // send read command 
      mmc_send_command (MMC_CMD_READBLOCK,
			HIGH (address >> 16),
			LOW (address >> 16), HIGH (address),
			LOW (address), 0xFF);

      // check if the MMC acknowledged the read block command 
      // it will do this by sending an affirmative response 
      // in the R1 format (0x00 is no errors) 
      if (mmc_wait_for_response (MMC_R1_RESPONSE) == MMC_SUCCESS)
	{
	  // now look for the data token to signify the start of
	  // the data 
	  if (mmc_wait_for_response (MMC_DATA_TOKEN) == MMC_SUCCESS)
	    {
	      // clock the actual data transfer and receive the bytes 
	      for (i = 0; i < block_length; ++i)
		{
		  mmc_in_buffer[i] = spi_read ();	// is executed with card inserted
		}

	      // get CRC bytes (not really needed by us, but required by MMC)
	      spi_read ();
	      spi_read ();
	    }
	  else
	    {
	      // the data token was never received 
	      rvalue = MMC_DATA_TOKEN_ERROR;	// 3
	    }
	}
      else
	{
	  // the MMC never acknowledge the read command 
	  rvalue = MMC_RESPONSE_ERROR;	// 2
	}

      // give the MMC the required clocks to 
      // finish up what ever it needs to do 
      spi_write (0xFF);
    }
  else
    {
      rvalue = MMC_BLOCK_SET_ERROR;	// 1
    }
  // CS = HIGH (off) 
  mmc_cs ();
  return;
}



//---------------------  first version: reduced copy of mmc_read_block, will not work  ------------------------------------------------
void
mmc_write_block (long int address)
{
//  unsigned long i = 0;
//  unsigned int rValue = MMC_SUCCESS;
//  address = block_length;

  unsigned long i = 0;
  unsigned int rValue = MMC_SUCCESS;

  // Set the block length to write (only possible with 512 Byte blocks)
  block_length = 512;
  if (mmc_set_block_length (block_length))
    {
      // SS = LOW (on) 
      mmc_cds ();

      // send write command 
      mmc_send_command (MMC_CMD_WRITEBLOCK,
			HIGH (address >> 16),
			LOW (address >> 16), HIGH (address),
			LOW (address), 0xFF);

      // check if the MMC acknowledged the write block command 
      // it will do this by sending an affirmative response 
      // in the R1 format (0x00 is no errors) 
      if (mmc_wait_for_response (MMC_R1_RESPONSE) == MMC_SUCCESS)
	{
	  // now look for the data token to signify the start of
	  // the data 
	  if (mmc_wait_for_response (MMC_DATA_TOKEN) == MMC_SUCCESS)
	    {
	      // clock the actual data transfer and receive the bytes 
	      for (i = 0; i < block_length; ++i)
		{
		  spi_write (mmc_out_buffer[i]);
		}

	      // get CRC bytes (not really needed by us, but required by MMC)
	      spi_read ();
	      spi_read ();
	    }
	  else
	    {
	      // the data token was never received 
	      rValue = MMC_DATA_TOKEN_ERROR;
	    }
	}
      else
	{
	  // the MMC never acknowledge the write command 
	  rValue = MMC_RESPONSE_ERROR;
	}

      // SS = HIGH (off) 
      mmc_cs ();

      // give the MMC the required clocks to 
      // finish up what ever it needs to do 
      spi_write (0xFF);
    }
  else
    {
      rValue = MMC_BLOCK_SET_ERROR;
    }
  return;
}



//---------------------------------------------------------------------
void
mmc_send_command (unsigned int cmd, unsigned int arg1, unsigned int
		  arg2, unsigned int arg3, unsigned int arg4,
		  unsigned int crc)
{
  // Send command 
  spi_write (cmd);

  // Send arguments 
  spi_write (arg1);
  spi_write (arg2);
  spi_write (arg3);
  spi_write (arg4);

  // Send CRC 
  spi_write (crc);
  return;
}

//---------------------------------------------------------------------
char
mmc_set_block_length (unsigned int block_length)
{
  char rValue = 1;

//  if (size != _BlockLength)
//    {
  // CS = LOW (on) 
  mmc_cds ();

  // Set the block length to read 
  mmc_send_command (MMC_CMD_SETLENGTH, 0, 0, HIGH (block_length),
		    LOW (block_length), 0xFF);

  // get response from MMC - make sure that its 0x00 (R1 ok response format)
  rValue = mmc_wait_for_response (MMC_R1_RESPONSE);

  // SS = HIGH (off) 
  mmc_cs ();

  // give the MMC the required clocks to 
  // finish up what ever it needs to do 
  spi_write (0xFF);
//    }

  return rValue;
}



void
spi_write (char x)
{
  while (!(IFG2 & UTXIFG1));
  TXBUF1 = x;
}



char
spi_read (void)
{
  spi_write (0xff);
  while (!(IFG2 & UTXIFG1));
  TXBUF1 = 0xff;
  return RXBUF1;
}


#endif /* _MMCLIB_C */
