/*
	mmc.h: declarations for communication with the MMC in SPI mode

	


Pin configuration (for MSP430F149):
-----------------------------------
  MC			MC Pin          MMC         	MMC Pin
  P5.4 			48 		ChipSelect	1
  P5.1 / SlaveOutMasterIn 45		DataOut 	2
  . 					GND 		3 (0 V)
  .					VDD 		4 (3.3 V)
  P5.3 UCLK1 / SlaveCLocK 47		Clock 		5
  .					GND 		6 (0 V)
  P5.2 / SlaveInMasterOut 46		DataIn 		7
---------------------------------------------------------------------


 Revisions
 Date         	Author            			Revision
 7. April 2003	nobodyo<nospam>@</nospam>web.de		0.01

*/


#ifndef _MMCLIB_H
#define _MMCLIB_H

// Defines:


// create desired types
#define bool short

// macro defines
#define MAX(a,b) (a>b?a:b)
#define MIN(a,b) (a<b?a:b)
#define HIGH(a) ((a>>8)&0xFF)
#define LOW(a) (a&0xFF)
#define EN_LOW() P5OUT &= ~0x01
#define EN_HIGH() P5OUT |= 0x01
#define SPI_RXC (IFG2 & URXIFG1)
#define SPI_TXC (IFG2 & UTXIFG1)

#define DUMMY 0xFF


// MMC Commands
// Set block length for next read/write
#define MMC_CMD_SETLENGTH 0x50	/* 0x50 = 80 */

// Write block to memory
#define MMC_CMD_WRITEBLOCK 0x54	/* 0x54 = 84 */

// Read block from memory
#define MMC_CMD_READBLOCK 0x51	/* 0x51 = 81 */

// Data token start byte
#define MMC_DATA_TOKEN 0xFE

// an affirmative R1 response (no errors)
#define MMC_R1_RESPONSE 0x00


// error/success codes
#define MMC_SUCCESS 0x00
#define MMC_BLOCK_SET_ERROR 0x01
#define MMC_RESPONSE_ERROR 0x02
#define MMC_DATA_TOKEN_ERROR 0x03
#define MMC_INIT_ERROR 0x04
#define MMC_TIMEOUT_ERROR 0xFF

// other stuff
#define MMC_GO_IDLE_STATE 0
#define MMC_SEND_OP_COND 1
#define MMC_SEND_CSD 9
#define MMC_SEND_CID 10
#define MMC_STOP_TRANSMISSION 12
#define MMC_SEND_STATUS 13
#define MMC_SET_BLOCKLEN 16
#define MMC_READ_SINGLE_BLOCK 17
#define MMC_READ_MULTIPLE_BLOCK 18
#define MMC_WRITE BLOCK 24
#define MMC_WRITE_MULTIPLE_BLOCK 25
#define MMC_PROGRAM_CSD 27
#define MMC_SET_WRITE_PROT 28
#define MMC_CLR_WRITE_PROT 29
#define MMC_SEND_WRITE_PROT 30
#define MMC_TAG_SECTOR_START 32
#define MMC_TAG_SECTOR_END 33
#define MMC_UNTAG_SECTOR 34
#define MMC_TAG_EREASE_GROUP_START 35
#define MMC_TAG_EREASE_GROUP_END 36
#define MMC_UNTAG_EREASE_GROUP 37
#define MMC_EREASE 38
#define MMC_READ_OCR 58
#define MMC_CRC_ON_OFF 59


static char mmc_in_buffer[16] = { 0 };
static char mmc_out_buffer[512] = { 0 };
static unsigned int block_length = 1;	// block length (for reading; writing only with 512 Byte)



// setup uart1 in spi mode
void uart1_spi_mode_init (void);



// Card Select
void mmc_cs (void);


// Card Deselect
void mmc_cds (void);


// SPI-Output (to card)
void spi_write (char);


// SPI-Input (to MC)
char spi_read (void);


// MMC block length
char mmc_set_block_length (unsigned int);


// command to MMC
void mmc_send_command (unsigned int cmd, unsigned int arg1, unsigned int
		       arg2, unsigned int arg3, unsigned int arg4,
		       unsigned int crc);


// wait for MMC response
char mmc_wait_for_response (char);


// initialize the MMC
char mmc_init (void);


// read a size (unsigned char) byte big block which begins at the address (unsigned long int).
void mmc_read_block (unsigned long int address, unsigned char count);


// write a 512 Byte big block, beginn at Address (unsigned long int)
void mmc_write_block (long int address);


#endif /* _MMCLIB_H */
